import 'package:example/todo/model/model.dart';
import 'package:flutter/foundation.dart';
import 'package:freezed_annotation/freezed_annotation.dart';
import 'package:mvu/mvu.dart';

import 'state.dart';
import 'update.dart';

part 'actions.freezed.dart';

@freezed
abstract class TodoListAction with _$TodoListAction {
  const factory TodoListAction.loading() = LoadingAction;

  const factory TodoListAction.loadingSucceeded({
    @required List<TodoItem> items,
  }) = LoadingSucceededAction;

  const factory TodoListAction.loadingFailed({
    @required dynamic error,
  }) = LoadingFailedAction;

  const factory TodoListAction.saving() = SavingAction;

  const factory TodoListAction.savingFailed({
    @required dynamic error,
  }) = SavingFailedAction;

  const factory TodoListAction.savingSucceeded() = SavingSucceededAction;

  const factory TodoListAction.updateItem({
    @required String itemId,
    @required bool isChecked,
  }) = UpdateItemAction;

  const factory TodoListAction.deleteItem({
    @required String itemId,
  }) = DeleteItemAction;

  const factory TodoListAction.createItem({
    @required String title,
  }) = CreateItemAction;

  static AsynchronousAction<TodoState> load() => AsynchronousAction(
        name: 'load',
        execute: TodoUpdate.load(),
      );

  static AsynchronousAction<TodoState> createItemAndSave({
    @required String title,
  }) =>
      AsynchronousAction(
        name: 'createItemAndSave',
        execute: TodoUpdate.createItemAndSave(
          title: title,
        ),
      );

  static AsynchronousAction<TodoState> updateItemAndSave({
    @required String itemId,
    @required bool isChecked,
  }) =>
      AsynchronousAction(
        name: 'updateItemAndSave',
        execute: TodoUpdate.updateItemAndSave(
          itemId: itemId,
          isChecked: isChecked,
        ),
      );
}
