import 'package:flutter/widgets.dart';
import 'package:provider/provider.dart';

typedef void Dispatcher(dynamic action);

typedef Widget ViewWidgetBuilder<Model>(
  BuildContext context,
  Model model,
);

typedef ViewSelector<Model, T> = T Function(Model model);

class View<Model, T> extends StatelessWidget {
  final ViewWidgetBuilder<T> builder;
  final ViewSelector<Model, T> selector;
  const View({
    Key key,
    @required this.selector,
    @required this.builder,
  }) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return Selector<Model, T>(
      selector: (context, model) => selector(model),
      builder: (context, value, _) => builder(context, value),
    );
  }
}
